=== modified file 'bzrlib/tests/HTTPTestUtil.py'
--- bzrlib/tests/HTTPTestUtil.py	2006-10-25 17:47:10 +0000
+++ bzrlib/tests/HTTPTestUtil.py	2006-12-11 13:24:55 +0000
@@ -170,3 +170,46 @@
     def setUp(self):
         super(TestCaseWithWebserver, self).setUp()
         self.transport_readonly_server = HttpServer
+
+
+class TestCaseWithTwoWebservers(TestCaseWithWebserver):
+    """A support class providinf readonly urls (on two servers) that are http://.
+
+    We setup two webservers to allows various tests involving
+    proxies or redirections from one server to the other.
+    """
+    def setUp(self):
+        super(TestCaseWithTwoWebservers, self).setUp()
+        self.transport_secondary_server = HttpServer
+        self.__secondary_server = None
+
+    def create_transport_secondary_server(self):
+        """Create a transport server from class defined at init.
+
+        This is mostly a hook for daughter classes.
+        """
+        return self.transport_secondary_server()
+
+    def get_secondary_server(self):
+        """Get the server instance for the secondary transport."""
+        if self.__secondary_server is None:
+            self.__secondary_server = self.create_transport_secondary_server()
+            self.__secondary_server.setUp()
+            # Note that addCleanup ensures that last added
+            # callables are called first. That will ensures that
+            # our cleanup is called before the other http server
+            # cleanup if we are created last and our cleanup is
+            # called after the other http server cleanup if we
+            # are created first.
+            self.addCleanup(self.__secondary_server.tearDown)
+        return self.__secondary_server
+
+
+class FakeProxyRequestHandler(TestingHTTPRequestHandler):
+    """Append a '-proxied' suffix to file served"""
+
+    def translate_path(self, path):
+        return TestingHTTPRequestHandler.translate_path(self,
+                                                        path + '-proxied')
+
+

=== modified file 'bzrlib/tests/HttpServer.py'
--- bzrlib/tests/HttpServer.py	2006-11-21 11:29:03 +0000
+++ bzrlib/tests/HttpServer.py	2006-12-11 13:24:55 +0000
@@ -248,8 +248,9 @@
     def _http_start(self):
         httpd = None
         httpd = self._get_httpd()
-        host, port = httpd.socket.getsockname()
-        self._http_base_url = '%s://localhost:%s/' % (self._url_protocol, port)
+        host, self.port = httpd.socket.getsockname()
+        self._http_base_url = '%s://localhost:%s/' % (self._url_protocol,
+                                                      self.port)
         self._http_starting.release()
         httpd.socket.settimeout(0.1)
 
@@ -287,19 +288,30 @@
         self._http_base_url = None
         self._http_thread = threading.Thread(target=self._http_start)
         self._http_thread.setDaemon(True)
-        self._http_thread.start()
+        # Ensure we are not influenced by the environment
         self._http_proxy = os.environ.get("http_proxy")
         if self._http_proxy is not None:
             del os.environ["http_proxy"]
+        self._no_proxy = os.environ.get("no_proxy")
+        if self._no_proxy is not None:
+            del os.environ["no_proxy"]
+        self._http_thread.start()
         self.logs = []
 
     def tearDown(self):
         """See bzrlib.transport.Server.tearDown."""
         self._http_running = False
         self._http_thread.join()
-        if self._http_proxy is not None:
-            import os
+        if self._http_proxy is None:
+            if os.environ.get("http_proxy") is not None:
+                del os.environ["http_proxy"]
+        else:
             os.environ["http_proxy"] = self._http_proxy
+        if self._no_proxy is None:
+            if os.environ.get("no_proxy") is not None:
+                del os.environ["no_proxy"]
+        else:
+            os.environ["no_proxy"] = self.no_proxy
 
     def get_url(self):
         """See bzrlib.transport.Server.get_url."""

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2006-11-21 01:08:52 +0000
+++ bzrlib/tests/test_http.py	2006-12-11 13:24:55 +0000
@@ -18,7 +18,9 @@
 # implementation; at the moment we have urllib and pycurl.
 
 # TODO: Should be renamed to bzrlib.transport.http.tests?
+# TODO: What about renaming to bzrlib.tests.transport.http ?
 
+import os
 import select
 import socket
 import threading
@@ -38,10 +40,12 @@
 from bzrlib.tests.HTTPTestUtil import (
     BadProtocolRequestHandler,
     BadStatusRequestHandler,
+    FakeProxyRequestHandler,
     ForbiddenRequestHandler,
     InvalidStatusRequestHandler,
     NoRangeRequestHandler,
     SingleRangeRequestHandler,
+    TestCaseWithTwoWebservers,
     TestCaseWithWebserver,
     WallRequestHandler,
     )
@@ -532,7 +536,7 @@
 
 
 class TestRangeRequestServer(object):
-    """Test the http connections.
+    """Tests readv requests against server.
 
     This MUST be used by daughter classes that also inherit from
     TestCaseWithWebserver.
@@ -546,8 +550,6 @@
         TestCaseWithWebserver.setUp(self)
         self.build_tree_contents([('a', '0123456789')],)
 
-    """Tests readv requests against server"""
-
     def test_readv(self):
         server = self.get_readonly_server()
         t = self._transport(server.get_url())
@@ -621,3 +623,83 @@
     """Tests range requests refusing server for pycurl implementation"""
 
 
+class TestProxyHttpServer(object):
+    """Tests proxy server.
+
+    This MUST be used by daughter classes that also inherit from
+    TestCaseWithTwoWebservers.
+
+    We can't inherit directly from TestCaseWithTwoWebservers or
+    the test framework will try to create an instance which
+    cannot run, its implementation being incomplete.
+    """
+
+    def setUp(self):
+        TestCaseWithTwoWebservers.setUp(self)
+        self.build_tree_contents([('foo', 'contents of foo\n'),
+                                  ('foo-proxied', 'proxied contents of foo\n')])
+        # Create the server now. Otherwise, it will created when
+        # test methods are invoked. At that point the test
+        # methods may have set env vars and the HttpServer
+        # creation will erase them.
+        server = self.get_readonly_server()
+        # The secondary server is the proxy
+
+        # NOTE: We do not setup a real proxy, instead we check
+        # that the connection goes thru the proxy by serving
+        # different content
+        proxy = self.get_secondary_server()
+        proxy_url = proxy.get_url()
+        # All tests will rely on the secondary server being set
+        # as the proxy
+        os.environ['http_proxy'] = proxy_url
+
+    def create_transport_secondary_server(self):
+        """Creates an http server that will serve files with
+        '-proxied' appended to their names.
+        """
+        return HttpServer(FakeProxyRequestHandler)
+
+    def test_http_proxy(self):
+        server = self.get_readonly_server()
+        url = server.get_url()
+        t = self._transport(url)
+        self.assertEqual(t.get('foo').read(),
+                         'proxied contents of foo\n')
+
+    def test_http_no_proxy(self):
+        server = self.get_readonly_server()
+        url = server.get_url()
+        os.environ['no_proxy'] = 'localhost:%d' % server.port
+        t = self._transport(url)
+        try:
+            self.assertEqual(t.get('foo').read(),
+                             'contents of foo\n')
+        finally:
+            del os.environ['no_proxy']
+
+
+class TestProxyHttpServer_urllib(TestProxyHttpServer,
+                                 TestCaseWithTwoWebservers):
+    """Tests proxy server for urllib implementation"""
+
+    _transport = HttpTransport_urllib
+
+
+class TestProxyHttpServer_pycurl(TestWithTransport_pycurl,
+                                 TestProxyHttpServer,
+                                 TestCaseWithTwoWebservers):
+    """Tests proxy server for pycurl implementation"""
+
+    def test_http_no_proxy(self):
+        server = self.get_readonly_server()
+        url = server.get_url()
+        # Oh my ! pycurl do not check the port part :-( So we
+        # just test the host part
+        os.environ['no_proxy'] = 'localhost'
+        t = self._transport(url)
+        try:
+            self.assertEqual(t.get('foo').read(),
+                             'contents of foo\n')
+        finally:
+            del os.environ['no_proxy']

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2006-12-08 16:44:33 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2006-12-11 13:24:55 +0000
@@ -662,28 +662,22 @@
 
     def proxy_bypass(self, host):
         """Check if host should be proxied or not"""
-        if self._debuglevel > 0:
-            print 'Is [%r] proxied or not' % host
         no_proxy = self.get_proxy('no')
         if no_proxy is None:
             return False
         hhost, hport = urllib.splitport(host)
-        if self._debuglevel > 0:
-            print 'hhost [%r], hport [%r]' % (hhost, hport)
         # Does host match any of the domains mentionned in
         # no_proxy The rules about what is authorized in no_proxy
         # are fuzzy (too say the least). We try to allows most
         # commonly seen values.
         for domain in no_proxy.split(','):
             dhost, dport = urllib.splitport(domain)
-            if self._debuglevel > 0:
-                print 'dhost [%r], dport [%r]' % (hhost, hport)
-            if hport == dport:
+            if hport == dport or dport is None:
                 # Protect glob chars
                 dhost = dhost.replace(".", r"\.")
                 dhost = dhost.replace("*", r".*")
                 dhost = dhost.replace("?", r".")
-                if re.match(domain, hhost, re.IGNORECASE):
+                if re.match(dhost, hhost, re.IGNORECASE):
                     return True
         # Nevertheless, there are platform specific ways to
         # ignore proxies...

